@@ -14,7 +14,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import datetime
+from collections import defaultdict
 import sys
 
 import sqlalchemy as sa
@@ -36,26 +36,36 @@ def get_backend():
 
 def _get_leases_from_resource_id(resource_id, start_date, end_date):
     session = get_session()
-    border0 = sa.and_(models.Lease.start_date < start_date,
-                      models.Lease.end_date < start_date)
-    border1 = sa.and_(models.Lease.start_date > end_date,
-                      models.Lease.end_date > end_date)
+    border0 = start_date <= models.Lease.end_date
+    border1 = models.Lease.start_date <= end_date
     query = (session.query(models.Lease).join(models.Reservation)
              .filter(models.Reservation.resource_id == resource_id)
-             .filter(~sa.or_(border0, border1)))
+             .filter(sa.and_(border0, border1)))
     for lease in query:
         yield lease
 
 
 def _get_leases_from_host_id(host_id, start_date, end_date):
+    session = get_session()
+    border0 = start_date <= models.Lease.end_date
+    border1 = models.Lease.start_date <= end_date
+    query = (session.query(models.Lease).join(models.Reservation)
+             .join(models.ComputeHostAllocation)
+             .filter(models.ComputeHostAllocation.compute_host_id == host_id)
+             .filter(sa.and_(border0, border1)))
+    for lease in query:
+        yield lease
+
+
+def _get_leases_from_fip_id(fip_id, start_date, end_date):
     session = get_session()
     border0 = sa.and_(models.Lease.start_date < start_date,
                       models.Lease.end_date < start_date)
     border1 = sa.and_(models.Lease.start_date > end_date,
                       models.Lease.end_date > end_date)
     query = (session.query(models.Lease).join(models.Reservation)
-             .join(models.ComputeHostAllocation)
-             .filter(models.ComputeHostAllocation.compute_host_id == host_id)
+             .join(models.FloatingIPAllocation)
+             .filter(models.FloatingIPAllocation.floatingip_id == fip_id)
              .filter(~sa.or_(border0, border1)))
     for lease in query:
         yield lease
@@ -63,46 +73,128 @@ def _get_leases_from_host_id(host_id, start_date, end_date):
 
 def get_reservations_by_host_id(host_id, start_date, end_date):
     session = get_session()
-    border0 = sa.and_(models.Lease.start_date < start_date,
-                      models.Lease.end_date < start_date)
-    border1 = sa.and_(models.Lease.start_date > end_date,
-                      models.Lease.end_date > end_date)
+    border0 = start_date <= models.Lease.end_date
+    border1 = models.Lease.start_date <= end_date
     query = (session.query(models.Reservation).join(models.Lease)
              .join(models.ComputeHostAllocation)
              .filter(models.ComputeHostAllocation.compute_host_id == host_id)
-             .filter(~sa.or_(border0, border1)))
+             .filter(sa.and_(border0, border1)))
     return query.all()
 
 
 def get_reservations_by_host_ids(host_ids, start_date, end_date):
     session = get_session()
-    border0 = sa.and_(models.Lease.start_date < start_date,
-                      models.Lease.end_date < start_date)
-    border1 = sa.and_(models.Lease.start_date > end_date,
-                      models.Lease.end_date > end_date)
+    border0 = start_date <= models.Lease.end_date
+    border1 = models.Lease.start_date <= end_date
     query = (session.query(models.Reservation).join(models.Lease)
              .join(models.ComputeHostAllocation)
              .filter(models.ComputeHostAllocation.compute_host_id
                      .in_(host_ids))
-             .filter(~sa.or_(border0, border1)))
+             .filter(sa.and_(border0, border1)))
     return query.all()
 
 
+def get_reservations_for_allocations(session, start_date, end_date,
+                                     lease_id=None, reservation_id=None):
+    fields = ['id', 'status', 'lease_id', 'start_date',
+              'end_date', 'lease_name', 'project_id']
+
+    reservations_query = (session.query(
+        models.Reservation.id,
+        models.Reservation.status,
+        models.Reservation.lease_id,
+        models.Lease.start_date,
+        models.Lease.end_date,
+        models.Lease.name,
+        models.Lease.project_id)
+        .join(models.Lease))
+
+    if lease_id:
+        reservations_query = reservations_query.filter(
+            models.Reservation.lease_id == lease_id)
+    if reservation_id:
+        reservations_query = reservations_query.filter(
+            models.Reservation.id == reservation_id)
+
+    # Only enforce time restrictions if we're not targeting a specific
+    # lease or reservation.
+    if not (lease_id or reservation_id):
+        border0 = models.Lease.end_date >= start_date
+        border1 = models.Lease.start_date <= end_date
+        reservations_query = reservations_query.filter(
+            sa.and_(border0, border1))
+
+    return [dict(zip(fields, r)) for r in reservations_query.all()]
+
+
+def get_reservation_allocations_by_host_ids(host_ids, start_date, end_date,
+                                            lease_id=None,
+                                            reservation_id=None):
+    session = get_session()
+    reservations = get_reservations_for_allocations(
+        session, start_date, end_date, lease_id, reservation_id)
+
+    allocations_query = (session.query(
+        models.ComputeHostAllocation.reservation_id,
+        models.ComputeHostAllocation.compute_host_id)
+        .filter(models.ComputeHostAllocation.compute_host_id.in_(host_ids))
+        .filter(models.ComputeHostAllocation.reservation_id.in_(
+            list(set([x['id'] for x in reservations])))))
+
+    allocations = defaultdict(list)
+
+    for row in allocations_query.all():
+        allocations[row[0]].append(row[1])
+
+    allocs = []
+    for r in reservations:
+        allocs.append((r['id'], r['lease_id'], allocations[r['id']][0]))
+
+    return allocs
+
+
+def get_reservation_allocations_by_fip_ids(fip_ids, start_date, end_date,
+                                           lease_id=None, reservation_id=None):
+    session = get_session()
+    reservations = get_reservations_for_allocations(
+        session, start_date, end_date, lease_id, reservation_id)
+
+    allocations_query = (session.query(
+        models.FloatingIPAllocation.reservation_id,
+        models.FloatingIPAllocation.floatingip_id)
+        .filter(models.FloatingIPAllocation.floatingip_id.in_(fip_ids))
+        .filter(models.FloatingIPAllocation.reservation_id.in_(
+            list(set([x['id'] for x in reservations])))))
+
+    allocations = defaultdict(list)
+
+    for row in allocations_query.all():
+        allocations[row[0]].append(row[1])
+
+    for r in reservations:
+        r['floatingip_ids'] = allocations[r['id']]
+
+    return reservations
+
+
 def get_plugin_reservation(resource_type, resource_id):
     if resource_type == host_plugin.RESOURCE_TYPE:
         return api.host_reservation_get(resource_id)
     elif resource_type == instance_plugin.RESOURCE_TYPE:
         return api.instance_reservation_get(resource_id)
     else:
-        raise mgr_exceptions.UnsupportedResourceType(resource_type)
+        raise mgr_exceptions.UnsupportedResourceType(
+            resource_type=resource_type)
 
 
-def get_free_periods(resource_id, start_date, end_date, duration):
+def get_free_periods(resource_id, start_date, end_date, duration,
+                     resource_type='host'):
     """Returns a list of free periods."""
     reserved_periods = get_reserved_periods(resource_id,
                                             start_date,
                                             end_date,
-                                            duration)
+                                            duration,
+                                            resource_type=resource_type)
     free_periods = []
     previous = (start_date, start_date)
     if len(reserved_periods) >= 1:
@@ -119,10 +211,17 @@ def get_free_periods(resource_id, start_date, end_date, duration):
     return free_periods
 
 
-def _get_events(host_id, start_date, end_date):
+def _get_events(resource_id, start_date, end_date, resource_type):
     """Create a list of events."""
     events = {}
-    for lease in _get_leases_from_host_id(host_id, start_date, end_date):
+    if resource_type == 'host':
+        leases = _get_leases_from_host_id(resource_id, start_date, end_date)
+    elif resource_type == 'floatingip':
+        leases = _get_leases_from_fip_id(resource_id, start_date, end_date)
+    else:
+        mgr_exceptions.UnsupportedResourceType(resource_type)
+
+    for lease in leases:
         if lease.start_date < start_date:
             min_date = start_date
         else:
@@ -184,86 +283,29 @@ def _merge_periods(reserved_periods, start_date, end_date, duration):
     return merged_reserved_periods
 
 
-def get_reserved_periods(host_id, start_date, end_date, duration):
-    """Returns a list of reserved periods for a host.
+def get_reserved_periods(resource_id, start_date, end_date, duration,
+                         resource_type='host'):
+    """Returns a list of reserved periods for a resource.
 
     The get_reserved_periods function returns a list of periods during which
-    the host passed as parameter is reserved. The duration parameter allows to
-    choose the minimum length of time for a period to be considered free.
+    the resource passed as parameter is reserved. The duration parameter
+    allows to choose the minimum length of time for a period to be
+    considered free.
 
-    :param host_id: the host to consider
+    :param resource_id: the resource to consider
     :param start_date: start datetime of the entire period to consider
     :param end_date: end datetime of the entire period to consider
     :param duration: minimum length of time for a period to be considered free
-    :returns: the list of reserved periods for the host, expressed as a list of
-              two-element tuples, where the first element is the start datetime
-              of the reserved period and the second is the end datetime
+    :param resource_type: A type of resource to consider
+    :returns: the list of reserved periods for the resource, expressed as a
+              list of two-element tuples, where the first element is the start
+              datetime of the reserved period and the second is
+              the end datetime
     """
     capacity = 1  # The resource status is binary (free or reserved)
     quantity = 1  # One reservation per host at the same time
     if end_date - start_date < duration:
         return [(start_date, end_date)]
-    events = _get_events(host_id, start_date, end_date)
+    events = _get_events(resource_id, start_date, end_date, resource_type)
     reserved_periods = _find_reserved_periods(events, quantity, capacity)
     return _merge_periods(reserved_periods, start_date, end_date, duration)
-
-
-def reservation_ratio(host_id, start_date, end_date):
-    res_time = reservation_time(host_id, start_date, end_date).seconds
-    return float(res_time) / (end_date - start_date).seconds
-
-
-def availability_time(host_id, start_date, end_date):
-    res_time = reservation_time(host_id, start_date, end_date)
-    return end_date - start_date - res_time
-
-
-def reservation_time(host_id, start_date, end_date):
-    res_time = datetime.timedelta(0)
-    for lease in _get_leases_from_host_id(host_id, start_date, end_date):
-        res_time += lease.end_date - lease.start_date
-        if lease.start_date < start_date:
-            res_time -= start_date - lease.start_date
-        if lease.end_date > end_date:
-            res_time -= lease.end_date - end_date
-    return res_time
-
-
-def number_of_reservations(host_id, start_date, end_date):
-    return sum(1 for x in
-               _get_leases_from_host_id(host_id, start_date, end_date))
-
-
-def longest_lease(host_id, start_date, end_date):
-    max_duration = datetime.timedelta(0)
-    longest_lease = None
-    session = get_session()
-    query = (session.query(models.Lease).join(models.Reservation)
-             .join(models.ComputeHostAllocation)
-             .filter(models.ComputeHostAllocation.compute_host_id == host_id)
-             .filter(models.Lease.start_date >= start_date)
-             .filter(models.Lease.end_date <= end_date))
-    for lease in query:
-        duration = lease.end_date - lease.start_date
-        if max_duration < duration:
-            max_duration = duration
-            longest_lease = lease.id
-    return longest_lease
-
-
-def shortest_lease(host_id, start_date, end_date):
-    # TODO(frossigneux) Fix max timedelta
-    min_duration = datetime.timedelta(365 * 1000)
-    longest_lease = None
-    session = get_session()
-    query = (session.query(models.Lease).join(models.Reservation)
-             .join(models.ComputeHostAllocation)
-             .filter(models.ComputeHostAllocation.compute_host_id == host_id)
-             .filter(models.Lease.start_date >= start_date)
-             .filter(models.Lease.end_date <= end_date))
-    for lease in query:
-        duration = lease.end_date - lease.start_date
-        if min_duration > duration:
-            min_duration = duration
-            longest_lease = lease.id
-    return longest_lease
